from terminal import Terminal
from nonTerminal import NonTerminal

class Grammar:
    def __init__(self):
        self.__nonTerminals = {}
        self.__terminals = {}
        self.__startSymbol = None;
        self.__endMarker = Terminal("$")

    def addProduction(self, nonTerminal, atoms):
        if self.__startSymbol == None:
            self.__startSymbol = nonTerminal;
            self.__startSymbol.addToFollowSet(self.__endMarker);

        self.__addAtom(nonTerminal);

        for atom in atoms:
            self.__addAtom(atom)

        key = nonTerminal.toString()
        self.__nonTerminals[key].addProduction(atoms)

    def __addAtom(self, atom):
        if isinstance(atom, Terminal):
            self.__addAtomToSet(self.__terminals, atom)
        elif isinstance(atom, NonTerminal):
            self.__addAtomToSet(self.__nonTerminals, atom)

    def __addAtomToSet(self, atomSet, atom):
        key = atom.toString()

        if atomSet.get(key) == None:
            atomSet[key] = atom


    def toString(self):
        retStr = ""

        for key in self.__nonTerminals:
            nonTerminal = self.__nonTerminals[key]

            retStr += nonTerminal.productionsToString() + "\n";

        return retStr

    def generateMetrics(self):
        self.__generateFirstSets()
        self.__generateFollowSets()


    def __generateFirstSets(self):
        for key in self.__nonTerminals:
            nonTerminal = self.__nonTerminals[key]
            nonTerminal.generateFirstSet();

    def __generateFollowSets(self):
        for key in self.__nonTerminals:
            self.__nonTerminals[key].generateFollowSet()

    def printNullableSet(self):
        output = "Nullable = { "

        for key in self.__nonTerminals:
            nonTerminal = self.__nonTerminals[key]

            if nonTerminal.isNullable():
                output += nonTerminal.toString() + " "

        output += "}"

        print(output)

    def printFirstSets(self):
        print("First Sets:")

        for key in self.__nonTerminals:
            nonTerminal = self.__nonTerminals[key]

            nonTerminal.printFirstSet();

        for key in self.__terminals:
            terminal = self.__terminals[key]

            terminal.printFirstSet();
